"use strict";

require("source-map-support/register");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var debug = require('debug')('metalsmith:markdown-parse');

var async = require('async');

var path = require('path');

var minimatch = require('minimatch');

var marked = require('marked');

var pug = require('pug');

var _require = require('@masala/parser'),
    Streams = _require.Streams;

var parse = require('./parse');

var _require2 = require('perf_hooks'),
    PerformanceObserver = _require2.PerformanceObserver,
    performance = _require2.performance;

var obs = new PerformanceObserver(function (items) {
  debug(items.getEntries()[0].name, items.getEntries()[0].duration);
});
obs.observe({
  entryTypes: ['measure']
});
/**
 * Expose `plugin`.
 */

module.exports = plugin;
/**
 *
 * Metalsmith plugin to parse markdown into file metadata.
 *
 * @param {Object} options
 * @param {string} options.pattern Pattern for navigation files.
 * @param {string} options.title Heading level 2 title to match for workflow (default: "Workflow").
 * @param {string} options.replace The whole workflow section will be replaced by this link,
 *                                 using the $start variable to link to the first question.
 *
 * @return {Function}
 */
// ## Workflow
//
// ### Workflow Label 1
// > Context
// Question Text
//   - [Answer Title](#workflow-label-2)
//   - [Answer Title](#workflow-label-3)

function plugin(options) {
  var _ref = options || {},
      _ref$pattern = _ref.pattern,
      pattern = _ref$pattern === void 0 ? '**/*.md' : _ref$pattern,
      _ref$title = _ref.title,
      title = _ref$title === void 0 ? 'Workflow' : _ref$title,
      _ref$replace = _ref.replace,
      replace = _ref$replace === void 0 ? '' : _ref$replace,
      _ref$layout = _ref.layout,
      layout = _ref$layout === void 0 ? 'question.pug' : _ref$layout;

  var parse_workflow = parse.workflow();
  var parse_workflow_raw = parse.workflow_raw();
  return function markdown_parse(files, metalsmith, done) {
    var tree = {},
        raw = {},
        processed = {};

    var process = function process(file, key, cb) {
      performance.mark('A'); // console.log('key', key);

      if (minimatch(key, pattern)) {
        var tokens = marked.lexer(file.contents.toString(), options); // console.log('tokens', tokens)

        var stream = Streams.ofArray(tokens); // const stream = Streams.ofArray([
        //   { type: "heading", text: "test" },
        //   { type: "blah", text: "a" }
        // ]);

        var parsing = parse_workflow.parse(stream); // console.log('parsing.value', JSON.stringify(parsing.value, true, 2))
        // the workflow_raw parser only splits by questions but parses what's between as raw content.

        var parsing_raw = parse_workflow_raw.parse(stream); // console.log(
        //   'parsing_raw.value',
        //   JSON.stringify(parsing_raw.value, true, 2)
        // )

        raw[key] = parsing_raw.value ? parsing_raw.value.array().map(function (question) {
          return question.reduce(function (_ref2, field) {
            var fields = _extends({}, _ref2);

            return _objectSpread({}, fields, {}, field);
          }, {});
        }) : []; // console.log('raw[' + key + ']', raw[key])

        var process_list = function process_list(list) {
          return list.map(function (item) {
            if (item.list) {
              return {
                type: 'html',
                text: marked.parser(Object.defineProperty(process_list(item.list), 'links', {
                  value: {}
                }))
              };
            } else {
              return item;
            }
          });
        };

        performance.mark('B');
        performance.measure('A to B: ' + key, 'A', 'B');

        try {
          processed[key] = parsing.value ? parsing.value.array().map(function (question) {
            // Render markdown inside parsed fields
            return question ? question.reduce(function (_ref3, field) {
              var fields = _extends({}, _ref3);

              // if (field.list) {
              //   console.log(
              //     'field.list',
              //     JSON.stringify(field.list, 2, true)
              //   )
              //   // field.list = field.list.filter(item => !item.list)
              //   console.log(
              //     'process_list(field.list)',
              //     JSON.stringify(process_list(field.list), 2, true)
              //   )
              //   console.log(
              //     'parser#',
              //     marked
              //       .parser(
              //         // field.list
              //         Object.defineProperty(
              //           process_list(field.list),
              //           'links',
              //           {
              //             value: {},
              //           }
              //         )
              //       )
              //       .replace(/<a href="#/g, '<a href="../')
              //   )
              // }
              return _objectSpread({}, fields, {}, field.quote ? {
                quote: marked.parser( // field.quote
                Object.defineProperty(field.quote, 'links', {
                  value: {}
                }))
              } : field.list ? {
                list: marked.parser(Object.defineProperty(process_list(field.list), 'links', {
                  value: {}
                })).replace(/<a href="#/g, '<a href="../')
              } : field.intro ? {
                description: marked(field.intro).trim()
              } : field.description ? {
                description: marked(field.description).trim()
              } : field.outro ? {
                description: marked(field.outro).trim()
              } : field);
            }, {}) : question;
          }) : [];
        } catch (e) {
          console.error(e);
          console.log('parsing.value.array()', parsing.value.array());
          throw e;
        } // console.log('processed[' + key + ']', processed[key])


        var unindent = function unindent(token) {
          return token.type === 'heading' && token.depth > 2 ? _objectSpread({}, token, {
            depth: token.depth - 2
          }) : token;
        }; // Add raw key with inter-heading blocks from workflow_raw


        tree[key] = processed[key].map(function (v, i) {
          return _objectSpread({}, v, {
            raw: marked.parser(Object.defineProperty(raw[key][i].value.map(unindent), 'links', {
              value: {}
            })) // transform local anchor links into link to individual question pages
            .replace(/<a href="#/g, '<a href="../')
          });
        }); // console.log('tree[' + key + ']', JSON.stringify(tree[key], true, 2))

        performance.mark('C');
        performance.measure('B to C: ' + key, 'B', 'C');
        tree[key].forEach(function (_ref4) {
          var index = _ref4.id,
              intro = _ref4.intro,
              description = _ref4.description,
              quote = _ref4.quote,
              list = _ref4.list,
              outro = _ref4.outro,
              _ref4$raw = _ref4.raw,
              raw = _ref4$raw === void 0 ? '' : _ref4$raw;
          // console.log('index', index)
          // console.log('question id', index)
          var newKey = key.replace('.md', '') + '/questions/' + index + '.md'; // Detect filtered collection transclusion link such as
          //  :[](organisations?tags=ddos,defaced&lang=en,es)

          var selector = /:<a href="(.*)"><\/a>/.exec(raw);

          var _ref5 = selector ? selector[1].split('?') : [],
              _ref6 = _slicedToArray(_ref5, 2),
              collection_name = _ref6[0],
              _ref6$ = _ref6[1],
              filter = _ref6$ === void 0 ? '' : _ref6$;

          var metadata = metalsmith.metadata();
          var collection = selector ? metadata[collection_name] : []; // console.log('filter', filter)
          // console.log('raw', raw)

          var processed = raw.replace(/<p>:<a href="(.*)"><\/a><\/p>/gm, collection.filter(function (_ref7) {
            var orglang = _ref7.language;
            return orglang === file.language;
          }).filter(function (item) {
            // console.log('item', item)
            return filter.split('&amp;').some(function (f) {
              // console.log('key', f.split('=')[0])
              // console.log('value', f.split('=')[1])
              // console.log('item[key]', item[f.split('=')[0]])
              // console.log(
              //   'true?',
              //   item[f.split('=')[0]]
              //     .split(',')
              //     .map(s => s.trim())
              //     .includes(f.split('=')[1])
              // )
              //
              try {
                var ret = f.split('=')[0] != '' ? item[f.split('=')[0]].split(',').map(function (s) {
                  return s.trim();
                }).includes(f.split('=')[1]) : true;
                return ret;
              } catch (e) {
                console.log(`Error while reading transclusion link in ${key}. Syntax should be \`:[](organisations?services=foo&services=bar)\` and found \`${filter.replace('&amp;', '&')}\` instead.`);
              }
            });
          }).sort(function (a, b) {
            var nameA = a.name.toUpperCase(); // ignore upper and lowercase

            var nameB = b.name.toUpperCase(); // ignore upper and lowercase

            if (nameA < nameB) {
              return -1;
            }

            if (nameA > nameB) {
              return 1;
            }

            return 0;
          }).map(function (item, index) {
            return pug.renderFile(path.join(metalsmith.dir || metalsmith._directory, metalsmith._src || metalsmith._source, '../code/templates', 'organisation.pug'), _objectSpread({}, item, {
              index
            }, metadata, {}, file.language ? {
              language: file.language
            } : null, {}, file.author ? {
              author: file.author
            } : null, {}, file.date ? {
              date: file.date
            } : null));
          }).join('\n')); // console.log('processed', processed);

          var newFile = _objectSpread({
            layout,
            intro,
            description,
            quote,
            list,
            outro,
            raw: processed,
            origin: key
          }, file.permalink ? {
            permalink: path.join(file.permalink, index)
          } : null, {
            contents: Buffer.from(processed // remove newline only paragraphs
            .replace('<p>\n</p>', ''))
          });

          files[newKey] = newFile;
        });
        performance.mark('D');
        performance.measure('C to D: ' + key, 'C', 'D'); // If tree[key] has content then we parsed a workflow and replace the section
        // console.log('file.contents', file.contents.toString())

        file.contents = tree[key][0] ? Buffer.from( // add end of file newline to match until Workflow end.
        (file.contents.toString() + '\n').replace(/## Workflow((\s|\S)*?)(.*\n)*/gm, replace.replace('$start', 'questions/' + tree[key][0].id) + '\n')) : file.contents; // console.log('file.contents', file.contents.toString());

        file[title.toLowerCase()] = tree[key].workflow; // console.log('file[' + title.toLowerCase() + ']', tree[key].workflow);
        // const parsed = md.parse(file.contents.toString());
        // console.log('parsed', JSON.stringify(parsed, true, 2));
        // return cb(null, file)

        performance.mark('E');
        performance.measure('D to E: ' + key, 'D', 'E');
        performance.measure('== A to E: ' + key, 'A', 'E');
        performance.clearMarks();
        return setImmediate(cb, null, file);
      }

      performance.mark('E');
      performance.measure('<> A to E: ' + key, 'A', 'E');
      performance.clearMarks();
      setImmediate(cb, null, file); // cb(null, file)
    };

    async.mapValuesSeries(files, process, function (err) {
      if (err) throw err;
      debug('Object.keys(files)', JSON.stringify(Object.keys(files), true, 2)); // Add #final_tips section in _end questions.

      Object.keys(files).filter(function (k) {
        return k.endsWith('_end.md');
      }).forEach(function (k) {
        var tips = k.split('/').slice(0, -1).join('/') + '/final_tips.md';

        if (files[tips]) {
          var arr_tips = files[tips].contents.toString().replace(/<h2 id="resources">Resources<\/h2>/gi, '<-----resource----->').split('<-----resource----->');
          files[k].tips = arr_tips[0];
          files[k].resources = arr_tips[1];
        } // delete files[tips]

      });
      setImmediate(done);
    });
  };
}
//# sourceMappingURL=index.js.map