"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.workflow_raw = exports.workflow = void 0;

require("source-map-support/register");

var _require = require('@masala/parser'),
    Parser = _require.Parser,
    _require$F = _require.F,
    not = _require$F.not,
    lazy = _require$F.lazy,
    Ftry = _require$F.try,
    response = _require.parsec.response; // const anyTokens = () =>
//   new Parser((input, index = 0) => {
//     // console.log("index", index);
//     const current = input.get(index);
//     return response.accept(current.value, input, index + 1, true);
//   });


var pattern = function pattern(obj) {
  return new Parser(function (input) {
    var index = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    // console.log("input.unsafeGet(index)", input.unsafeGet(index));
    var current = input.get(index);

    if (current && current.value && Object.keys(obj).every(function (k1) {
      return Object.keys(current.value).some(function (k2) {
        return obj[k1] === current.value[k2];
      });
    })) {
      return response.accept(current.value, input, index + 1, true);
    }

    return response.reject(input, index, false);
  });
};

var space = function space() {
  return pattern({
    type: 'space'
  });
};

var spaces = function spaces() {
  return space().optrep().drop();
};

var paragraph = function paragraph(key) {
  return function () {
    return pattern({
      type: 'paragraph'
    }) // .debug("paragraph")
    .then(spaces()).map(function (_ref) {
      var text = _ref.text;
      return {
        [key]: text
      };
    });
  };
}; // const description = () =>
//   pattern({ type: 'paragraph' })
//     // .debug("paragraph")
//     .then(spaces())
//     .map(({ text }) => ({ description: text }))


var blockquote = function blockquote() {
  return pattern({
    type: 'blockquote_start'
  }).drop().then(not(pattern({
    type: 'blockquote_end'
  })).rep().map(function (_ref2) {
    var value = _ref2.value;
    return value;
  })).then(pattern({
    type: 'blockquote_end'
  }).drop()).map(function (tokens) {
    return Array.isArray(tokens) ? {
      quote: tokens
    } : {
      quote: [tokens]
    };
  });
};

var list = function list() {
  return pattern({
    type: 'list_start'
  }).then(Ftry(lazy(list)).or(not(pattern({
    type: 'list_end'
  }))).rep().map(function (_ref3) {
    var value = _ref3.value;
    return value;
  })).then(pattern({
    type: 'list_end'
  })).map(function (tokens) {
    return {
      list: tokens
    };
  });
};

var content = function content() {
  return spaces().then(paragraph('intro')().optrep().map(function (_ref4) {
    var value = _ref4.value;
    return value;
  })).then(spaces()) // .debug('intro')
  .then(blockquote().optrep().map(function (_ref5) {
    var value = _ref5.value;
    return value;
  })) // .debug('blockquote')
  .then(spaces()).then(paragraph('description')().optrep().map(function (_ref6) {
    var value = _ref6.value;
    return value;
  })) // .debug('description')
  .then(spaces()).then(list().optrep().map(function (_ref7) {
    var value = _ref7.value;
    return value;
  })) // .debug('list')
  .then(spaces()).then(paragraph('outro')().optrep().map(function (_ref8) {
    var value = _ref8.value;
    return value;
  })) // .debug('outro')
  .then(spaces()).then(not(pattern({
    type: 'heading',
    depth: 3
  })).optrep().map(function (_ref9) {
    var value = _ref9.value;
    return value;
  })).then(spaces());
};

var heading = function heading() {
  return pattern({
    type: 'heading',
    depth: 2,
    text: 'Workflow'
  }).drop();
};

var question = function question() {
  return pattern({
    type: 'heading',
    depth: 3
  }).map(function (_ref10) {
    var text = _ref10.text;
    return {
      id: text
    };
  }).then(content()).map(function (r) {
    return [r];
  });
};

var workflow = function workflow() {
  return not(heading()).optrep().drop().then(heading()) // .debug('after heading')
  .then(question().rep());
}; // .debug('after question rep')
// It might be possible to do this in only one pass


exports.workflow = workflow;

var question_raw = function question_raw() {
  return pattern({
    type: 'heading',
    depth: 3
  }).map(function (_ref11) {
    var text = _ref11.text;
    return {
      id: text
    };
  }).then(not(pattern({
    type: 'heading'
  }).filter(function (heading) {
    return heading.depth <= 3;
  })).rep()).map(function (r) {
    return [r];
  });
};

var workflow_raw = function workflow_raw() {
  return not(heading()).optrep().drop().then(heading()).then(question_raw().rep());
};

exports.workflow_raw = workflow_raw;
//# sourceMappingURL=parse.js.map